#!/usr/bin/env ruby
#
# CS 430 OO Ruby Example
#

# Animal (base class)
#
#   - attributes: species, color
#   - methods: observe(), to_s()
#   - operators: ==, <=>
#
class Animal

  attr_accessor :species
  attr_accessor :color

  # equivalent to "attr_accessor :species":
  #  def species
  #    return @species
  #  end
  #  def species=(value)
  #    @species = value
  #  end

  def initialize (species, color)
    @species = species
    @color = color
  end

  def observe
    return "You see a #{@color} #{@species}"
  end

  def to_s
    return observe
  end

  def ==(other)
    return @species == other.species &&
           @color == other.color
  end

  def <=>(other)
    @species == other.species ? @color <=> other.color : @species <=> other.species
  end

end


# Fish "is a" Animal
#
#   - new method: swim()
#
class Fish < Animal
  def swim
    puts "A #{@species} is swimming!"
  end
end


# Mammal "is a" Animal
#
#   - new attribute: num_legs
#   - new method: run()
#
class Mammal < Animal

  attr_accessor :num_legs

  def initialize(species, color, num_legs)
    super(species, color)
    @num_legs = num_legs
  end

  def run
    puts "A #{@num_legs}-legged #{@species} is running!"
  end

end


# Dog "is a" Mammal
#
#   - new attribute: name
#   - new method: bark()
#   - overloaded method: observe()
#
class Dog < Mammal
  def initialize(breed, name)
    super("dog", breed, 4)
    @name = name
  end

  def bark
    puts "#{@name} barks!"
  end

  def observe
    return "#{super()} named #{@name}"
  end
end


donald = Animal.new("duck", "blue")
puts(donald.observe())
puts(donald)
gets

nemo = Fish.new("clownfish", "orange")
puts(nemo)
nemo.swim()
gets

marlin = Fish.new("clownfish", "orange")
puts(marlin)
marlin.swim()
puts(marlin == donald)
puts(marlin == nemo)
gets

seabiscuit = Mammal.new("horse", "light bay", 4)
puts(seabiscuit)
seabiscuit.run()
gets

cornelius = Mammal.new("chimpanzee", "brown", 2)
puts(cornelius)
cornelius.run()
gets

lassie = Dog.new("collie", "Lassie")
puts(lassie)
lassie.run()
lassie.bark()
gets

puts([donald, nemo, marlin, seabiscuit, cornelius, lassie].sort)
